from distutils.core import setup
setup(
  name = 'subseek',
  packages = ['subseek'],
  version = '2.0.6',
  description = 'Search subtitles for all your movies and series with a single command line script',
  author = 'Federico Nicolas Motta',
  author_email = 'fedemotta@gmail.com',
  url = 'https://github.com/fedemotta/subseek',
  download_url = 'https://github.com/fedemotta/subseek/tarball/2.0.6',
  keywords = ['subtitles', 'movies', 'series','batch','script'],
  classifiers = [],
)